@@ -263,7 +263,7 @@ This option forces pasrsehttp() method to use a L<HTTP::Lite> instance.
 
 =head2 lwp_useragent
 
-This option forces pasrsehttp() method to use a L<LWP::UserAgent> instance.
+This option forces parsehttp() method to use a L<LWP::UserAgent> instance.
 
     my $ua = LWP::UserAgent->new();
     $ua->timeout( 60 );
@@ -309,6 +309,14 @@ this module dereferences the predefined character entity references,
 &amp;, &lt;, &gt;, &apos; and &quot;, and the numeric character
 references up to U+007F without xml_deref per default.
 
+=head2 require_xml_decl
+
+This option requires XML declaration at the top of XML document to parse.
+
+    $tpp->set( require_xml_decl => 1 );
+
+This will die when <?xml .../?> declration not found.
+
 =head1 OPTIONS FOR WRITING XML
 
 =head2 first_out
@@ -354,6 +362,12 @@ encodings supported by Encode.pm. On Perl 5.6.x and before with
 Jcode.pm, you can use C<Shift_JIS>, C<EUC-JP>, C<ISO-2022-JP> and
 C<UTF-8>. The default value is C<UTF-8> which is recommended encoding.
 
+=head2 empty_element_tag_end
+
+    $tpp->set( empty_element_tag_end => '>' );
+
+Set characters which close empty tag. The default value is ' />'.
+
 =head1 OPTIONS FOR BOTH
 
 =head2 utf8_flag
@@ -405,11 +419,18 @@ This makes parsing performance slow.
 
 Yusuke Kawasaki, http://www.kawa.net/
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT
 
-Copyright (c) 2006-2009 Yusuke Kawasaki. All rights reserved.
-This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+The following copyright notice applies to all the files provided in
+this distribution, including binary files, unless explicitly noted
+otherwise.
+
+Copyright 2006-2010 Yusuke Kawasaki
+
+=head1 LICENSE
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =cut
 
@@ -419,7 +440,7 @@ use Carp;
 use Symbol;
 
 use vars qw( $VERSION );
-$VERSION = '0.39';
+$VERSION = '0.42';
 
 my $XML_ENCODING      = 'UTF-8';
 my $INTERNAL_ENCODING = 'UTF-8';
@@ -429,6 +450,8 @@ my $TEXT_NODE_KEY     = '#text';
 my $USE_ENCODE_PM     = ( $] >= 5.008 );
 my $ALLOW_UTF8_FLAG   = ( $] >= 5.008001 );
 
+my $EMPTY_ELEMENT_TAG_END = ' />';
+
 sub new {
     my $package = shift;
     my $self    = {@_};
@@ -510,7 +533,6 @@ sub write {
     my $apre = $self->{attr_prefix} if exists $self->{attr_prefix};
     $apre = $ATTR_PREFIX unless defined $apre;
     local $self->{__attr_prefix_len} = length($apre);
-#    local $self->{__attr_prefix_rex} = defined $apre ? qr/^\Q$apre\E/s : undef;
     local $self->{__attr_prefix_rex} = $apre;
 
     local $self->{__indent};
@@ -632,7 +654,7 @@ sub parsehttp_lwp {
     if ( defined $body && ! $ct ) {
         $req->header( 'Content-Type' => 'application/x-www-form-urlencoded' );
     }
-    $req->content($body) if defined $body;
+    $req->add_content_utf8($body) if defined $body;
     my $res = $ua->request($req);
     my $code = $res->code();
     my $text;
@@ -722,6 +744,11 @@ sub parse {
         return $self->die( "Tie::IxHash is required." ) unless &load_tie_ixhash();
     }
 
+    # Avoid segfaults when receving random input (RT #42441)
+    if ( exists $self->{require_xml_decl} && $self->{require_xml_decl} ) {
+        return $self->die( "XML declaration not found" ) unless looks_like_xml(\$text);
+    }
+
     my $flat  = $self->xml_to_flat(\$text);
     my $class = $self->{base_class} if exists $self->{base_class};
     my $tree  = $self->flat_to_tree( $flat, '', $class );
@@ -968,6 +995,8 @@ sub hash_to_xml {
     }
     my $prelen = $self->{__attr_prefix_len};
     my $pregex = $self->{__attr_prefix_rex};
+    my $textnk = $self->{text_node_key};
+    my $tagend = $self->{empty_element_tag_end} || $EMPTY_ELEMENT_TAG_END;
 
     foreach my $keys ( $firstkeys, $allkeys, $lastkeys ) {
         next unless ref $keys;
@@ -977,7 +1006,8 @@ sub hash_to_xml {
         foreach my $key ( @$elemkey ) {
             my $val = $hash->{$key};
             if ( !defined $val ) {
-                push( @$out, "<$key />" );
+                next if ($key eq $textnk);
+                push( @$out, "<$key$tagend" );
             }
             elsif ( UNIVERSAL::isa( $val, 'HASH' ) ) {
                 my $child = $self->hash_to_xml( $key, $val );
@@ -1021,7 +1051,7 @@ sub hash_to_xml {
             $text = "<$name$jattr>$text</$name>\n";
         }
         else {
-            $text = "<$name$jattr />\n";
+            $text = "<$name$jattr$tagend\n";
         }
     }
     $text;
@@ -1032,9 +1062,11 @@ sub array_to_xml {
     my $name  = shift;
     my $array = shift;
     my $out   = [];
+    my $tagend = $self->{empty_element_tag_end} || $EMPTY_ELEMENT_TAG_END;
+
     foreach my $val (@$array) {
         if ( !defined $val ) {
-            push( @$out, "<$name />\n" );
+            push( @$out, "<$name$tagend\n" );
         }
         elsif ( UNIVERSAL::isa( $val, 'HASH' ) ) {
             my $child = $self->hash_to_xml( $name, $val );
@@ -1101,10 +1133,19 @@ sub read_raw_xml {
     $text;
 }
 
+sub looks_like_xml {
+    my $textref = shift;
+    my $args = ( $$textref =~ /^(?:\s*\xEF\xBB\xBF)?\s*<\?xml(\s+\S.*)\?>/s )[0];
+    if ( ! $args ) {
+        return;
+    }
+    return $args;
+}
+
 sub xml_decl_encoding {
     my $textref = shift;
     return unless defined $$textref;
-    my $args    = ( $$textref =~ /^(?:\s*\xEF\xBB\xBF)?\s*<\?xml(\s+\S.*)\?>/s )[0] or return;
+    my $args    = looks_like_xml($textref) or return;
     my $getcode = ( $args =~ /\s+encoding=(".*?"|'.*?')/ )[0] or return;
     $getcode =~ s/^['"]//;
     $getcode =~ s/['"]$//;
